#include <qpixmap.h>
#include <qstyle.h>
#include <qpainter.h>
#include <qpushbutton.h>
#include <qimage.h>
#include <qstylefactory.h>
#include <qapplication.h>
#include <qtabbar.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qscrollbar.h>
#include <qmenubar.h>
#include <qmenudata.h>
#include <qcombobox.h>

#include <stdlib.h>

#include <gtk/gtknotebook.h>
#include <gtk/gtkgc.h>

#include "qt_qt_wrapper.h"


QScrollBar* meepBar;  // Scrollbars are strange beasts - we only use one instance
QWidget* meepWidget;
QValueList<CachedImage> cache;  // A cache of widget pixmaps to speed up drawing
QPixmap* backgroundTile;

CachedImage::CachedImage(WidgetType t, GtkStateType s, int w, int h, QImage i)
{
	type = t;
	state_type = s;
	width = w;
	height = h;
	image = i;
}

CachedImage::CachedImage(WidgetType t, GtkStateType s, int w, int h, QImage i, int extra)
{
	type = t;
	state_type = s;
	width = w;
	height = h;
	image = i;
	if ((t == CheckBox) || (t == RadioButton)) on = extra;
	if (t == LineEdit) hasFocus = extra;
}

void cacheAppend(CachedImage image)
{
	while (cache.count() > 50)
		cache.remove(cache.begin());

	cache.append(image);
}


void createQApp()
{
	int argc = 0;
	char** argv = (char**) malloc(sizeof(char*));
	argv[0] = (char*) malloc(sizeof(char));
	argv[0][0] = '\0';

	new QApplication(argc, argv);
	meepBar = new QScrollBar(NULL);
	meepWidget = new QWidget(NULL);
	meepWidget->polish();
	backgroundTile = (QPixmap*)meepWidget->paletteBackgroundPixmap();

	delete argv;
}

void destroyQApp()
{
	delete meepBar;
	delete meepWidget;
	delete qApp;
}



// The drawing functions follow the same pattern:
//  * Check for a cached pixmap of this widget (if one exists, use it)
//  * Set the appropriate flags
//  * Ask QT to paint the widget to a pixmap
//  * Convert the pixmap to a BGR image
//  * Add this image to the cache
//  * Ask GDK to paint the image to the window


void drawButton(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int x, int y, int width, int height)
{
	QValueList<CachedImage>::iterator it;
    for ( it = cache.begin(); it != cache.end(); ++it )
    {
		if (((*it).type == CachedImage::Button) &&
			((*it).state_type == state_type) &&
			((*it).width == width) &&
			((*it).height == height))
		{
			gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, (*it).image.bits(), (*it).image.bytesPerLine());
			return;
		}
	}

	QPixmap* pixmap = new QPixmap(width,height);
	QPainter* painter = new QPainter(pixmap);
	QPushButton* button = new QPushButton(NULL);

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	switch (state_type)
	{
	case GTK_STATE_ACTIVE: 		sflags |= QStyle::Style_Down;		break;
	case GTK_STATE_PRELIGHT:	sflags |= QStyle::Style_MouseOver;	break;
	case GTK_STATE_SELECTED:	sflags |= QStyle::Style_HasFocus;	break;
	case GTK_STATE_INSENSITIVE:	sflags = QStyle::Style_Default;		break;
	}

	qApp->style().drawControl(QStyle::CE_PushButton, painter, button, QRect(0,0,width,height), qApp->palette().active(), sflags);

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;
	delete button;

	cacheAppend(CachedImage(CachedImage::Button, state_type, width, height, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawTab(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int x, int y, int width, int height)
{
	QValueList<CachedImage>::iterator it;
    for ( it = cache.begin(); it != cache.end(); ++it )
    {
		if (((*it).type == CachedImage::Tab) &&
			((*it).state_type == state_type) &&
			((*it).width == width) &&
			((*it).height == height))
		{
			gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, (*it).image.bits(), (*it).image.bytesPerLine());
			return;
		}
	}

	QPixmap* pixmap = new QPixmap(width,height);
	QPainter* painter = new QPainter(pixmap);
	QWidget* tabbarparent = new QWidget(); // Included to stop some styles from complaining
	QTabBar* tabbar = new QTabBar(tabbarparent);
	QTab* tab = new QTab("");

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	if (state_type != GTK_STATE_ACTIVE)
		sflags = QStyle::Style_Selected;

	painter->fillRect(0,0,width,height,qApp->palette().active().background());
	qApp->style().drawControl(QStyle::CE_TabBarTab, painter, tabbar, QRect(0,0,width,height), qApp->palette().active(), sflags, QStyleOption(tab));

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;
	delete tabbar;
	delete tabbarparent;
	delete tab;

	cacheAppend(CachedImage(CachedImage::Tab, state_type, width, height, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawVLine(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int x, int y1, int y2)
{
	int width = 2;
	int height = style->xthickness;

	QPixmap* pixmap = new QPixmap(width, height);
	QPainter* painter = new QPainter(pixmap);

	painter->fillRect(2,0,width-2,height,qApp->palette().active().background());
	painter->setPen( qApp->palette().active().mid() );
	painter->drawLine( 0, 0, 0, height );
	painter->setPen( qApp->palette().active().light() );
	painter->drawLine( 1, 0, 1, height );

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y1, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawHLine(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int y, int x1, int x2)
{
	int width = abs(x2-x1);
	int height = style->ythickness;

	QPixmap* pixmap = new QPixmap(width, height);
	QPainter* painter = new QPainter(pixmap);

	painter->fillRect(0,2,width,height-2,qApp->palette().active().background());
	painter->setPen( qApp->palette().active().mid() );
	painter->drawLine( 0, 0, width, 0);
	painter->setPen( qApp->palette().active().light() );
	painter->drawLine( 0, 1, width, 1);

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x1, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawLineEdit(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int hasFocus, int x, int y, int width, int height)
{
	QValueList<CachedImage>::iterator it;
    for ( it = cache.begin(); it != cache.end(); ++it )
    {
		if (((*it).type == CachedImage::LineEdit) &&
			((*it).state_type == state_type) &&
			((*it).width == width) &&
			((*it).height == height) &&
			((*it).hasFocus == hasFocus))
		{
			gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, (*it).image.bits(), (*it).image.bytesPerLine());
			return;
		}
	}

	QPixmap* pixmap = new QPixmap(width, height);
	QPainter* painter = new QPainter(pixmap);

	QStyle::SFlags sflags = QStyle::Style_Enabled;
	if (hasFocus)
		sflags = QStyle::Style_HasFocus;

	painter->fillRect(0,0,width,height,qApp->palette().active().background());
	qApp->style().drawPrimitive(QStyle::PE_PanelLineEdit, painter, QRect(0,0,width,height), qApp->palette().active(), sflags, QStyleOption(1,1));

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;

	cacheAppend(CachedImage(CachedImage::LineEdit, state_type, width, height, image, hasFocus));

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawFrame(GdkWindow * window, GtkStyle * style, GtkStateType state_type, GtkShadowType shadow_type, int x, int y, int width, int height)
{
	QValueList<CachedImage>::iterator it;
    for ( it = cache.begin(); it != cache.end(); ++it )
    {
		if (((*it).type == CachedImage::Frame) &&
			((*it).state_type == state_type) &&
			((*it).width == width) &&
			((*it).height == height))
		{
			gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, (*it).image.bits(), (*it).image.bytesPerLine());
			return;
		}
	}

	QPixmap* pixmap = new QPixmap(width, height);
	QPainter* painter = new QPainter(pixmap);

	QStyle::SFlags sflags = QStyle::Style_Enabled;
	if ((shadow_type == GTK_SHADOW_IN) || (shadow_type == GTK_SHADOW_ETCHED_IN))
		sflags |= QStyle::Style_Sunken;

	if ((backgroundTile) && (!backgroundTile->isNull()))
		painter->fillRect(0,0,width,height,QBrush(QColor(255,255,255), *backgroundTile));
	else
		painter->fillRect(0,0,width,height,qApp->palette().active().background());

	qApp->style().drawPrimitive(QStyle::PE_Panel, painter, QRect(0,0,width,height), qApp->palette().active(), sflags, QStyleOption(1,1) );

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;

	cacheAppend(CachedImage(CachedImage::Frame, state_type, width, height, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawComboBox(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int x, int y, int width, int height)
{
	QValueList<CachedImage>::iterator it;
    for ( it = cache.begin(); it != cache.end(); ++it )
    {
		if (((*it).type == CachedImage::ComboBox) &&
			((*it).state_type == state_type) &&
			((*it).width == width) &&
			((*it).height == height))
		{
			gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, (*it).image.bits(), (*it).image.bytesPerLine());
			return;
		}
	}

	QPixmap* pixmap = new QPixmap(width, height);
	QPainter* painter = new QPainter(pixmap);
	QComboBox* cb = new QComboBox(false, NULL);

	QStyle::SFlags sflags = QStyle::Style_Enabled;
	QStyle::SCFlags scflags = QStyle::SC_ComboBoxArrow | QStyle::SC_ComboBoxFrame | QStyle::SC_ComboBoxListBoxPopup;
	QStyle::SCFlags activeflags = QStyle::SC_None;

	switch(state_type)
	{
	case GTK_STATE_ACTIVE:		sflags |= QStyle::Style_HasFocus; 		break;
	case GTK_STATE_PRELIGHT:	activeflags = QStyle::SC_ComboBoxArrow;	break;
	}

	painter->fillRect(0,0,width,height,qApp->palette().active().background());
	qApp->style().drawComplexControl(QStyle::CC_ComboBox, painter, cb, QRect(0,0,width,height), qApp->palette().active(), sflags, scflags, activeflags);

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;
	delete cb;

	cacheAppend(CachedImage(CachedImage::ComboBox, state_type, width, height, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawCheckBox(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int on, int x, int y, int width, int height)
{
	QValueList<CachedImage>::iterator it;
    for ( it = cache.begin(); it != cache.end(); ++it )
    {
		if (((*it).type == CachedImage::CheckBox) &&
			((*it).state_type == state_type) &&
			((*it).width == width) &&
			((*it).height == height) &&
			((*it).on == on))
		{
			gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, (*it).image.bits(), (*it).image.bytesPerLine());
			return;
		}
	}

	QPixmap* pixmap = new QPixmap(width,height);
	QPainter* painter = new QPainter(pixmap);
	QCheckBox* checkbox = new QCheckBox(NULL);

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	switch (state_type)
	{
	case GTK_STATE_ACTIVE: 		sflags = QStyle::Style_Down;		break;
	case GTK_STATE_PRELIGHT:	sflags = QStyle::Style_MouseOver;	break;
	case GTK_STATE_SELECTED:	sflags = QStyle::Style_HasFocus;	break;
	case GTK_STATE_INSENSITIVE:	sflags = QStyle::Style_Default;		break;
	}

	if (on)
		sflags |= QStyle::Style_On;
	else
		sflags |= QStyle::Style_Off;

	painter->fillRect(0,0,width,height,qApp->palette().active().background());
	qApp->style().drawControl(QStyle::CE_CheckBox, painter, checkbox, QRect(0,0,width,height), qApp->palette().active(), sflags);

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;
	delete checkbox;

	cacheAppend(CachedImage(CachedImage::CheckBox, state_type, width, height, image, on));

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawRadioButton(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int on, int x, int y, int width, int height)
{
	QValueList<CachedImage>::iterator it;
    for ( it = cache.begin(); it != cache.end(); ++it )
    {
		if (((*it).type == CachedImage::RadioButton) &&
			((*it).state_type == state_type) &&
			((*it).width == width) &&
			((*it).height == height) &&
			((*it).on == on))
		{
			gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, (*it).image.bits(), (*it).image.bytesPerLine());
			return;
		}
	}

	QPixmap* pixmap = new QPixmap(width,height);
	QPainter* painter = new QPainter(pixmap);
	QRadioButton* radio = new QRadioButton(NULL);

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	switch (state_type)
	{
	case GTK_STATE_ACTIVE: 		sflags = QStyle::Style_Down;		break;
	case GTK_STATE_PRELIGHT:	sflags = QStyle::Style_MouseOver;	break;
	case GTK_STATE_SELECTED:	sflags = QStyle::Style_HasFocus;	break;
	case GTK_STATE_INSENSITIVE:	sflags = QStyle::Style_Default;		break;
	}

	if (on)
		sflags |= QStyle::Style_On;
	else
		sflags |= QStyle::Style_Off;

	painter->fillRect(0,0,width,height,qApp->palette().active().background());
	qApp->style().drawControl(QStyle::CE_RadioButton, painter, radio, QRect(0,0,width,height), qApp->palette().active(), sflags);

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;
	delete radio;

	cacheAppend(CachedImage(CachedImage::RadioButton, state_type, width, height, image, on));

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawScrollBar(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int orientation, GtkAdjustment* adj, int x, int y, int width, int height)
{
	//QScrollBar* sbar = new QScrollBar(0,100,1,10,50,orientation ? Qt::Vertical : Qt::Horizontal, NULL);

	meepBar->setMinValue((int)adj->lower);
	meepBar->setMaxValue((int)adj->upper - adj->page_size);
	meepBar->setValue((int)adj->value);
	meepBar->setPageStep((int)adj->page_size);
	meepBar->setOrientation(orientation ? Qt::Vertical : Qt::Horizontal);
	meepBar->resize(width,height);

	QImage image = QPixmap::grabWidget(meepBar).convertToImage().swapRGB();

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawMenuBarItem(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int x, int y, int width, int height)
{
	QValueList<CachedImage>::iterator it;
    for ( it = cache.begin(); it != cache.end(); ++it )
    {
		if (((*it).type == CachedImage::MenuItem) &&
			((*it).state_type == state_type) &&
			((*it).width == width) &&
			((*it).height == height))
		{
			gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, (*it).image.bits(), (*it).image.bytesPerLine());
			return;
		}
	}

	QPixmap* pixmap = new QPixmap(width,height);
	QPainter* painter = new QPainter(pixmap);
	QMenuItem* mi = new QMenuItem();
	QMenuBar* mb = new QMenuBar(NULL);

	QStyle::SFlags sflags = QStyle::Style_Active | QStyle::Style_HasFocus;

	switch (state_type)
	{
	case GTK_STATE_ACTIVE: 		sflags |= QStyle::Style_Down;		break;
	case GTK_STATE_PRELIGHT:	sflags |= QStyle::Style_MouseOver;	break;
	case GTK_STATE_SELECTED:	sflags |= QStyle::Style_HasFocus;	break;
	case GTK_STATE_INSENSITIVE:	sflags = QStyle::Style_Default;		break;
	}

	qApp->style().drawControl(QStyle::CE_MenuBarItem, painter, mb, QRect(0,0,width,height), qApp->palette().active(), sflags, QStyleOption(mi));

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;
	delete mi;
	delete mb;

	cacheAppend(CachedImage(CachedImage::MenuItem, state_type, width, height, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawMenuItem(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int x, int y, int width, int height)
{
	gdk_draw_rectangle(window, style->bg_gc[state_type], true, x, y, width, height);
}

void drawSplitter(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int orientation, int x, int y, int width, int height)
{
	QValueList<CachedImage>::iterator it;
    for ( it = cache.begin(); it != cache.end(); ++it )
    {
		if (((*it).type == CachedImage::Splitter) &&
			((*it).state_type == state_type) &&
			((*it).width == width) &&
			((*it).height == height))
		{
			gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, (*it).image.bits(), (*it).image.bytesPerLine());
			return;
		}
	}

	QPixmap* pixmap = new QPixmap(width, height);
	QPainter* painter = new QPainter(pixmap);

	QStyle::SFlags sflags = QStyle::Style_Enabled;
	if (orientation == 1)
		sflags = QStyle::Style_Horizontal;

	painter->fillRect(0,0,width,height,qApp->palette().active().background());
	qApp->style().drawPrimitive(QStyle::PE_Splitter, painter, QRect(0,0,width,height), qApp->palette().active(), sflags);

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;

	cacheAppend(CachedImage(CachedImage::Splitter, state_type, width, height, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawTabFrame(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int x, int y, int width, int height)
{
	QValueList<CachedImage>::iterator it;
    for ( it = cache.begin(); it != cache.end(); ++it )
    {
		if (((*it).type == CachedImage::TabFrame) &&
			((*it).state_type == state_type) &&
			((*it).width == width) &&
			((*it).height == height))
		{
			gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, (*it).image.bits(), (*it).image.bytesPerLine());
			return;
		}
	}

	QPixmap* pixmap = new QPixmap(width, height);
	QPainter* painter = new QPainter(pixmap);

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	if ((backgroundTile) && (!backgroundTile->isNull()))
		painter->fillRect(0,0,width,height,QBrush(QColor(255,255,255), *backgroundTile));
	else
		painter->fillRect(0,0,width,height,qApp->palette().active().background());
	qApp->style().drawPrimitive(QStyle::PE_TabBarBase, painter, QRect(0,0,width,height), qApp->palette().active(), sflags);

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;

	cacheAppend(CachedImage(CachedImage::TabFrame, state_type, width, height, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawMenu(GdkWindow * window, GtkStyle * style, GtkStateType state_type, int x, int y, int width, int height)
{
	QValueList<CachedImage>::iterator it;
    for ( it = cache.begin(); it != cache.end(); ++it )
    {
		if (((*it).type == CachedImage::Menu) &&
			((*it).state_type == state_type) &&
			((*it).width == width) &&
			((*it).height == height))
		{
			gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, (*it).image.bits(), (*it).image.bytesPerLine());
			return;
		}
	}

	QPixmap* pixmap = new QPixmap(width, height);
	QPainter* painter = new QPainter(pixmap);

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	painter->fillRect(0,0,width,height,qApp->palette().active().background());
	qApp->style().drawPrimitive(QStyle::PE_PanelPopup, painter, QRect(0,0,width,height), qApp->palette().active(), sflags, QStyleOption(1,1));

	QImage image = pixmap->convertToImage().swapRGB();
	delete painter;
	delete pixmap;

	cacheAppend(CachedImage(CachedImage::Menu, state_type, width, height, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state_type], x, y, width, height, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void setColors(GtkStyle* style)
{
	bool useBg = ((backgroundTile) && (!backgroundTile->isNull()));
	QImage image;

	if (useBg)
		image = backgroundTile->convertToImage().swapRGB();

	for (uint i=0 ; i<5 ; i++)
	{
		switch (i)
		{
		case GTK_STATE_NORMAL:
		case GTK_STATE_ACTIVE:
		case GTK_STATE_INSENSITIVE:
			style->bg[i].red = qApp->palette().active().background().red()*257;
			style->bg[i].green = qApp->palette().active().background().green()*257;
			style->bg[i].blue = qApp->palette().active().background().blue()*257;
			break;
		case GTK_STATE_PRELIGHT:
		case GTK_STATE_SELECTED:
			style->bg[i].red = qApp->palette().active().highlight().red()*257;
			style->bg[i].green = qApp->palette().active().highlight().green()*257;
			style->bg[i].blue = qApp->palette().active().highlight().blue()*257;
			break;
		}

		if (i == GTK_STATE_SELECTED)
		{
			style->base[i].red = qApp->palette().active().highlight().red()*257;
			style->base[i].green = qApp->palette().active().highlight().green()*257;
			style->base[i].blue = qApp->palette().active().highlight().blue()*257;
			style->text[i].red = qApp->palette().active().highlightedText().red()*257;
			style->text[i].green = qApp->palette().active().highlightedText().green()*257;
			style->text[i].blue = qApp->palette().active().highlightedText().blue()*257;
		}
		else
		{
			style->base[i].red = qApp->palette().active().base().red()*257;
			style->base[i].green = qApp->palette().active().base().green()*257;
			style->base[i].blue = qApp->palette().active().base().blue()*257;
			style->text[i].red = qApp->palette().active().text().red()*257;
			style->text[i].green = qApp->palette().active().text().green()*257;
			style->text[i].blue = qApp->palette().active().text().blue()*257;
		}

		style->fg[i].red = qApp->palette().active().foreground().red()*257;
		style->fg[i].green = qApp->palette().active().foreground().green()*257;
		style->fg[i].blue = qApp->palette().active().foreground().blue()*257;
	}

	if (useBg)
	{
		GdkPixmap* pixmap = gdk_pixmap_new(NULL, image.width(), image.height(), style->depth);
		GdkGCValuesMask gc_values_mask = GDK_GC_FOREGROUND;
		GdkGCValues gc_values;
		gc_values.foreground = style->fg[0];
		GdkGC* gc = gtk_gc_get (style->depth, style->colormap, &gc_values, gc_values_mask);
		gdk_draw_rgb_32_image(pixmap, gc, 0, 0, image.width(), image.height(), GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
		if (pixmap) style->bg_pixmap[GTK_STATE_NORMAL] = pixmap;
		gtk_gc_release(gc);
	}
}
